//
//
// JUMA-TRX2 Rotary Encoder routines
//
//
//  Juha Niinikoski OH2NLT 24.01.2006
//
// Microchip C30 version
//
// Trigger from also encoder phase B added 26.11.2006
// Encoder coarse read added 04.12.2006
// Very fine back and forth tuning anomaly corrected 21.12.2006
// Changed to every edge sensitive 26.12.2006 



//Includes
#include "juma-trx2.h"

// rotary encoder
int enc_accu;					// encoder pulse accumulator
unsigned char	enc_event;		// encoder event happened
static int last_phase;			// last processed phase, A=0, B=1

// Encoder IRQ routines
// very quick IQ encoder handler, no debounce, needs clean signal edges

// Encoder Phase A / RA15 / INT4
void __attribute__((interrupt)) _INT4Interrupt(void)
	{
	_INT4EP = !_INT4EP;			// change sensitive edge
	_INT4IF = 0;
	enc_event = 1;
	if(last_phase != 0)			// decode only if previous event was phase B
		{
		if(_INT4EP == 0)		// rising edge action
			{
			if(ENQ_B)
				enc_accu++;
			else
				enc_accu--;
			}
		else					// falling edge action
			{
			if(ENQ_B)
				enc_accu--;
			else
				enc_accu++;
			}

		}
	last_phase = 0;				// mark phase A
	}

// Encoder Phase B / RA14 / INT3
void __attribute__((interrupt)) _INT3Interrupt(void)
	{
	_INT3EP = !_INT3EP;			// invert trigger polarity
	_INT3IF = 0;
	enc_event = 1;
	if(last_phase == 0)			// decode only if previous event was phase A
		{
		if(_INT3EP == 0)		// rising edge action
			{
			if(ENQ_A)
				enc_accu--;
			else
				enc_accu++;
			}
		else					// falling edge action
			{
			if(ENQ_A)
				enc_accu++;
			else
				enc_accu--;
			}
		}
	last_phase = 1;				// mark phase B
	}


// Rotary Encoder system init
void encoder_init(void)
	{
//	ICP9 = 0x0010;				// set priority to 1, default = 4
	_INT4IF = 0;
	_INT4IE = 1;
	_INT3IF = 0;
	_INT3IE = 1;			
	}

int encoder_get(void)			// return with encoder reading
	{
	int x;
	x = enc_accu;
	enc_accu = 0;
	enc_event = 0;
	return(x);
	}

int encoder_get_coarse(int steps) // return with +/-1 if moved over asked stepd
	{
	int x;
	if(enc_accu > steps)
		{
		enc_accu = 0;
		enc_event = 0;
		return(1);
		}

	if(enc_accu < (0 - steps))
		{
		enc_accu = 0;
		enc_event = 0;
		return(-1);
		}

	return(0);			// return with 0 if not moved
	}
