//
// JUMA-TRX2 Receiver Controller & DDS
// Juha Niinikoski, OH2NLT 22.11.2006
//
// 
// MPLAB C30 version
//
// MCU dsPIC30F6014A
//
// Project started 22.11.2006
// User interface & calibration displays 01.12.2006
// Slpit & RIT logic 02.12.2006
// Main board 74HC595 SPI interface added 03.11.2006
// Keyer code added 08.12.2006
// Keyer operating mede select added 10.12.2006
// Power off button timer & factory default load 12.12.2006
// Button tones 13.12.2006
// SWR calculations 19.12.2006
// all sort of user interface modifications, EEPROM checksum 26.12.2006
// service mode separated to different module 27.12.2006
// SWR alarm, fwd voltage bar graph, tune start beep 30.12.2006
// Yaesu CAT emulator, first test version 12.01.2007
// VFO lock function added 18.01.2007
// Iambic mode B added to keyer code 27.01.2007
// EEPROM structure separated to own trx2_eeprom.h file 28.01.2007
// ISD17240 Voice memory code tests 11.03.2007
// external kb module added 12.03.2007
// ISD17240 Voice memory installed flag, more memory banks & user interface changes 17.03.2007
// ISD17240 Voice memory Rev.B board definition changes 21.03.2007
// S-meter scale improvement. S9 marker and end stopper addes 03.04.2007
// Squelch code 08.04.2007
// All Band PA filter select logic & user interface tones improved 15.04.2007
// PA RF attenuator test 23.04.2007
// PA RF filter & attenuator selector code 17.05.2007
// MODE stored separatelly for both VFO's 17.05.2005
// PA RF attenuator (Power level) bits to EEPROM save 02.06.2007
// Different upper frequency limit for normal and service mode 03.06.2007
// MODE and VFO changes disabled when TX on, when Split selected Keyer and Tune operation corrected 06.06.2007
// Version changed to 1.00, first release package 19.06.2007
// RS232 test loop added 09.07.2007

const char ver[] = {"090707"};				// software version information, displayed at startup

#include "juma-trx2.h"						// JUMA-TRX2 hardware specific definitions
#include "lcd-trx2.h"						// LCD specific definitions
#include "DataEEPROM.h"


_FOSC(CSW_FSCM_OFF & EC);					// 30MHz clock= 7,5MHz cycle
_FWDT(WDT_OFF);                				//Turn off the Watch-Dog Timer.
_FBORPOR(MCLR_EN & PWRT_64 & BORV_20);   	//Enable MCLR, power on timer, brown out 2V
//_FGS(GEN_PROT);            				//Disable Code Protection, old compiler
 _FBS(CODE_PROT_OFF);
 _FSS(CODE_PROT_OFF);
 _FGS(CODE_PROT_OFF);


// Global variabbles

// DDS & clock variables
unsigned long ref_osc;			// actual oscilator frequency, calibrated value
long max_dds;					// max DDS frequency, different value for normal use and service mode
int fcy;						// clock / kHz

// tuning
#define STEPS 6
// raw tuning steps
//const long f_steps[STEPS] = {1, 100, 1000,  10000, 100000, 1000000};	// increment / decrement steps

// frequency decade steps for 180,000 MHz master clock
const long f_steps[STEPS] = {48, 477, 4772, 47722, 477219, 4772186};	// 1, 10, 100, 1000, 10000, 100000 Hz
int step_idx;					// step table index
#define MAX_STEP_IDX 5			// step table size

// global DDS variables
long long vltemp;				// very long variable for frequency display calculations
long freq_in_use;				// frequency (DDS set value) currently in use
int vfo_idx;					// VFO index, A=0, B=1, if(Split) then  RX=0, TX=1
long rit;						// rit +/- adder

// multi band PA filter select, setup transmitted via SPI bus to filter board
unsigned int rf_fselect;																// selected filter number
const long rf_filters[7] = {K250, K500, M1, M2, M4, M8, M15};							// filter change points, test version
unsigned int rf_attn;			// PA RF attenuator, use only D14 and D15, others must be zero

#define	RF_FILT_TABLE_LEN 11
const long rf_filt_attn[RF_FILT_TABLE_LEN] = {K250, K500, M1, M2, M4, M8 ,M12, M15, M19, M23, M26};	// filter change points

const unsigned int rf_filter_bits[RF_FILT_TABLE_LEN+1] = {
0x0000, 0x0100, 0x0200, 0x0300, 0x0400, 0x0500, 0x0600, 0x0600, 0x0700, 0x0700,  0x0700, 0x0700};

int rf_filt_idx;				// index to filter & attenuator bits table
unsigned char rf_attn_test;	// test output from selector

// system status flags
int cw_active;					// status flag for keyer module
int tune_on;					// tune state indicator
int PTT_IN;						// atomic copy of PTT_IN_IO input / TX on, 0 = active

// timing
int seconds;					// free running 1s timer, used by ISD1700 routines

// keyer
int keyer_mode;					// 0=dot priority, 1=iambic, 2=straight

// voice memory option
int isd_1700_installed;			// voice memory board found

// display control
#define MAX_LCD_MODE 1			// last display "main" page
//#define MAX_LCD_MODE 2			// last display "main" page / test display included
#define	MAX_SUB_PAGE0 4			// sub page 0 displays
#define	MAX_SUB_PAGE1 9			// sub page 1 displays
#define S_BAR 0x07				// S-meter vertical bar character

unsigned char lcdpbuff[20];		// LCD print buffer
int	lcd_mode;					// lcd display mode
int sub_page0;					// lcd display sub page for lcd_mode0
int sub_page1;					// lcd display sub page for lcd_mode1
int service_mode;				// service mode selected flag
int vfo_lock;					// VFO knob tune lock

// ADC & meter variables
int batt;						// Powersupply voltage scaled value, measured only when displayed
long batt_mean_val;
int bat_samples = 0;
int s_meter;					// scaled S-meter value 0 ... 48
int fwd_adval;					// forward & reverse power raw values, measured only when TX
int rev_adval;
int id_cur;						// PA Drain current raw value

long fwd_pwr;					// relative forward power
long rev_pwr;					// relative reverse power
long out_pwr;					// scaled output power = fwd_pwr - rev_pwr * scale
int swr;						// calculated swr * 100
int swr_alarm;					//alarm flag
int swr_al_dly;					// swr alarm activation delay counter, main loop rounds
int pwr_meter;					// scaled S-meter value 0 ... 48

int cw_pot;						// CW speed potentiometer, also used for squelch

int sq_timer;					// squelch open timer




// SCAF filter
const int scaf_min[3] = {NAR_SCAF, MID_SCAF, MIN_SCAF};		// filter tuning limits
const int scaf_max[3] = {MAX_SCAF, NAR_SCAF, MID_SCAF};


// User power on values, EEPROM storage 
unsigned int fd_counter;			// counts how many times factory defaults have reloaded

#include "trx2_eeprom.h"				// EEPROM storage structure

union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

// system calibration values

union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;


// external functions

extern void us_delay( unsigned int);				// delay routines
extern void ms_delay( unsigned int);

extern void init_timers_pwm(void);					// Timers & PWM DAC
extern void init_scaf_clock(void);					// start scaf clocking
extern void stop_scaf_clock(void);					// stop scaf clocking
extern void start_scaf_clock(void);					// start scaf clocking
extern void set_scaf_clock(unsigned int);			// scaf filter clock divider
extern void set_pwm3_dac(unsigned int);				// LCD back light
extern void set_pwm4_dac(unsigned int);				// LCD contrast

extern void tone_on(int);							// start tone generator, period length
extern void tone_off(void);							// stop tone generator
extern void beep(int, int);							// generate beep, period, length

extern void dds_load(unsigned long set_val);
extern void dds_init(void);

extern void init_adc12(void);
extern int convert_adc12(unsigned int adchs);		// ADC

extern void draw_s_meter(int);						// LCD functions
extern void draw_bar(int, int);
extern void lcdoutch(unsigned char);
extern void lcd_putst(register const char *);
extern void set_chgen(void);
extern void initlcd(void);
extern void clear_lcd(void);
extern void set_cur_lcd(unsigned char);
extern void disp_freq(long);
extern void disp_rit(int);
extern void disp_meter(int, int);
extern void disp_bcd(char);
extern void lcd_putchhex(unsigned char);

extern void InitUSART1(void);						// terminal functions
extern void putch(char);
extern int kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);
extern void putlong( unsigned long);

extern void init_spi1(void);						// SPI, main board 74HC595 I/O latch
extern void exchg_data_spi1(unsigned int);

extern void encoder_init(void);						// VFO encoder functions
extern int encoder_get(void);						
extern int encoder_get_coarse(int);

extern void service(void);							// service functions

extern void serial_test(void);						// serial I/O
extern void dump_eeprom(void);						// dump EEPROM content
extern void serial_yaesu(void);						// Yaesu CAT emulator
extern void serial_ext_kb(void);					// external keyboard (Voice Memory control)

extern  void init_isd1700(void);					// Init ISD1700 voice memory system


//external variables

extern int enc_accu;								// VFO encoder
extern unsigned char enc_event;						// VFO encoder moved flag
extern unsigned int long_push;						// long push timer for buttons
extern int cw_period;
extern int cw_break_timer;	

// Main board 74HC595 I/O latch
unsigned int hc595;									// latch bits defined in juma-trx2.h file


// Helper functions

// Set factory defaults for EEPROM storage

void set_factory_defaults(void)
	{
// user settings
	eeprom.defval.vfo[0] = DEFAULT_FREQ_A;
	eeprom.defval.vfo[1] = DEFAULT_FREQ_B;
	eeprom.defval.step_idx = 1;
	eeprom.defval.vfo_sel = 0;
	eeprom.defval.mode[0] = 0;
	eeprom.defval.mode[1] = 0;
	eeprom.defval.rit_on = 0;
	eeprom.defval.agc_speed = 0;
	eeprom.defval.proc_on_off = 0;
	eeprom.defval.nb = 0;
	eeprom.defval.filter = 2;						// WID, for Matti
	eeprom.defval.contrast = DEFAULT_CONTRAST;
	eeprom.defval.back_light = DEFAULT_BL;
	eeprom.defval.scaf_filters[0] = DEFAULT_NAR;
	eeprom.defval.scaf_filters[1] = DEFAULT_MID;
	eeprom.defval.scaf_filters[2] = DEFAULT_WID;
	eeprom.defval.scaf_filters[3] = DEFAULT_TX;
	eeprom.defval.cw_shift = CW_SHIFT;
	eeprom.defval.keyer = KEYER;
	eeprom.defval.serial_mode = SERIAL_MODE;
	eeprom.defval.af_input = MIC;
	eeprom.defval.sq = 0;							// squels not in use

// calibration values
	cal.calval.ref_osc = REF_OSC;
	cal.calval.id_mult = ID_MULT;
	cal.calval.batt_mult = BATT_MULT;
	cal.calval.s_mtr_mult = S_MTR_MULT;
	cal.calval.rev_pwr_mult = REV_PWR_MULT;
	cal.calval.fwd_pwr_mult = FWD_PWR_MULT;
	cal.calval.beep_len = DEFAULT_BEEP_LEN;
	cal.calval.rf_atn_bits[0] = 3;					// 250 kHz  init values for RF attenuator bits
	cal.calval.rf_atn_bits[1] = 3;					// 500 kHz  valid range 0 ... 3
	cal.calval.rf_atn_bits[2] = 1;					// 1 MHz
	cal.calval.rf_atn_bits[3] = 1;					// 2 MHz
	cal.calval.rf_atn_bits[4] = 1;					// 4 MHz
	cal.calval.rf_atn_bits[5] = 1;					// 8 MHz
	cal.calval.rf_atn_bits[6] = 1;					// 12 MHz
	cal.calval.rf_atn_bits[7] = 1;					// 15 MHz
	cal.calval.rf_atn_bits[8] = 1;					// 19 MHz
	cal.calval.rf_atn_bits[9] = 1;					// 23 MHz
	cal.calval.rf_atn_bits[10] =1;					// 26 MHz
	cal.calval.rf_atn_bits[11] =1;					// 26 MHz

// increment factory default reset counter
	ReadEE(EEPAGE, EE_FD_LOC, &fd_counter, WORD);
	fd_counter++;
// save factory defaults reset counter
	EraseEE(EEPAGE, EE_FD_LOC, WORD);
	WriteEE(&fd_counter, EEPAGE, EE_FD_LOC, WORD);
	}


// Save defaults

void save_defaults(void)
	{
	int x,y;

	eeprom.defval.d_csum = 0;
	for(x=0; x<(sizeof(struct defval)/2); x++)
		{
		if(x < ((sizeof(struct defval)/2) - 1))
			eeprom.defval.d_csum = eeprom.defval.d_csum + eeprom.storage[x]; // calculate mod 0xFFFF checksum
		else
			eeprom.defval.d_csum = 0 - eeprom.defval.d_csum;			// last round, invert checksum

 		y = EraseEE(EEPAGE, (2*x+EEDEF), WORD);
		y = WriteEE(&(eeprom.storage[x]), EEPAGE, (2*x+EEDEF), WORD);	// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		}
	printf("\n\rWords = %d, Sucess = %d, Checksum = %X", x, y, eeprom.defval.d_csum);	// test
	}

// Read defaults

unsigned int read_defaults(void)
	{
	int x,y;
	unsigned int csum;

	csum = 0;
	for(x=0; x<(sizeof(struct defval)/2); x++)
		{
		y = ReadEE(EEPAGE, (2*x+EEDEF), &(eeprom.storage[x]), WORD);	// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
// add error checking ?  y = success code

		csum = csum + eeprom.storage[x];								
		}

// read factory defaults reset counter
	ReadEE(EEPAGE, EE_FD_LOC, &fd_counter, WORD);

	return csum;
	}


// Save calibration values

void save_calval(void)
	{
	int x,y;

	cal.calval.c_csum = 0;
	for(x=0; x<(sizeof(struct calval)/2); x++)
		{
		if(x < ((sizeof(struct calval)/2) - 1))
			cal.calval.c_csum = cal.calval.c_csum + cal.ee[x]; // calculate mod 0xFFFF checksum
		else
			cal.calval.c_csum = 0 - cal.calval.c_csum;

		y = EraseEE(EEPAGE, (2*x+EECAL), WORD);
		y = WriteEE(&(cal.ee[x]), EEPAGE, (2*x+EECAL), WORD);			// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		}
	printf("\n\rWords = %d, Success = %d, Checksum = %X", x, y, cal.calval.c_csum);	// test
	}


// Read calibration values

unsigned int read_calval(void)
	{
	int x,y;
	unsigned int csum;

	csum = 0;
	for(x=0; x<(sizeof(struct calval)/2); x++)
		{
		y = ReadEE(EEPAGE, (2*x+EECAL), &(cal.ee[x]), WORD);			// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		csum = csum + cal.ee[x];
		}
	return csum;
	}


// calculate SWR

int calc_swr(int fwd, int rev)
	{
//
// Ro = sqrt(Pr/Pf) or Ro = Ur/Uf
// SWR = (1+Ro)/(1-Ro)
//
	int swr;						// calculated swr
	long Ro100;						// 100*Ro
	long x;							// 100*(1-Ro)

	Ro100 = ((long)rev*100L) / fwd;	//  rev/fwd ratio * 100
	x = 100 - Ro100;
	if(x < 1)
		x = 1;						// prevent divide by zero

	if(fwd >= 50)					// calculate only if we have some signal
		swr = (((100 + Ro100)*100) / x);	// 100 * (1+Ro)/(1-Ro)
	else
		swr = 0;					// no swr reading if low power

	return(swr);					// value 100 = swr 1.00
	}


// calculate & display bar graph S-meter

void display_s_meter(void)
	{
	unsigned long ultemp;
	ultemp = (unsigned long)convert_adc12(S_METER) * cal.calval.s_mtr_mult;		// convert & scale S-meter
	s_meter = ultemp >> 16;														// div by 65536, 2V = full scale(48) @ default cal
	draw_s_meter(s_meter);														// S-meter displayed when AGC speed is adjusted
	}

// calculate & display bar graph power (forward voltage) meter

void display_pwr_meter(void)
	{
	int temp;
//	pwr_meter = (out_pwr * PWR_GRAPH_MULT) >> 20;		// forward power display
	pwr_meter = (fwd_adval * F_VOLT_GRAPH_MULT) >> 16; 	// forward voltage display
	draw_s_meter(pwr_meter);							// full bar = 10W
	}

// display bar graph meter, S-meter when RX, PWR when TX

void disp_bar_meter(void)
	{
	if(PTT_IN == 0)									// TXon ?
		display_pwr_meter();						// display power meter
	else
		display_s_meter();							// display S-meter
	}

// find RF filter & attenuator select index
// global variable rf_filt_idx = result
void find_rf_filt_idx(void)
	{
	int x;
	for(x=0; x<RF_FILT_TABLE_LEN; x++)				// scan filter & attenuator switch point table
		{
		if(freq_in_use < rf_filt_attn[x])
			break;		
		}
	rf_filt_idx = x;								// set resolved index
	}



// Build main board & RF filter board(only all band version) 74HC595 I/O latch data & send it

// D14, D15 reserved for PA board attenuator
// D8,D9,D10 RF filter select bits

void send_packet_to_74hc595(void)
		{
		find_rf_filt_idx();								// resolve rf_filt_idx
		rf_attn = ((unsigned int)cal.calval.rf_atn_bits[rf_filt_idx] << 14);	// get attenuator bits

		hc595 = rf_filter_bits[rf_filt_idx];			// set filter select bits
		hc595 = hc595 | rf_attn;						// combine with attenuator bits

		if(freq_in_use > M7_FILT)						// D7, 3,5 / 7 MHZ filter select(for two band PA)
				hc595 = hc595 | MHZ7;


		if(eeprom.defval.af_input != LINE)				// D6, MIC/Line input level
				hc595 = hc595 | AF_INPUT;

		if(eeprom.defval.nb != 0)						// D5, AGC speed
				hc595 = hc595 | NB;


		if(eeprom.defval.proc_on_off != 0 && eeprom.defval.af_input == MIC)	// D4, Speech proc
				hc595 = hc595 | PROC_ON;


		if(eeprom.defval.agc_speed != 0)				// D3, AGC speed
				hc595 = hc595 | FAST_AGC;


		if(eeprom.defval.filter == 0)					// D2, Narrow SSB/CW band pass filter select
				hc595 = hc595 | NAR;

		if(eeprom.defval.mode[vfo_idx] != 1)			// D1, sideband select, USB = 1, LSB & CW & Tune = 0
				hc595 = hc595 | SB_SELECT;

		if(eeprom.defval.mode[vfo_idx] < 2)				// D0, SSB / !CW select
				hc595 = hc595 | SSB_CW;

		exchg_data_spi1(hc595);							// start transmission to main board lathc
		}

// function for Multiband PA input RF attenuator addjust
void addjust_rf_atttn(void)
	{
	int temp;

	while(PWR_SW == 1)										// stay here while PWR switch is pressed
		{
// do A/D conversions
		fwd_adval = convert_adc12(FWD_PWR);
		rev_adval = convert_adc12(REV_PWR);
// calculate Power
		fwd_pwr = (long)fwd_adval * (long)fwd_adval;		// calculate relative power levels ( U^2)
		rev_pwr = (long)rev_adval * (long)rev_adval;
		out_pwr = fwd_pwr - rev_pwr;						// output power = forward - reverse.

// draw meters
		set_cur_lcd(LINE1);
		disp_bar_meter();									// display S-meter or PWR meter bar graph
		lcd_putst("PWR");
		temp = (out_pwr * cal.calval.fwd_pwr_mult) >> 16;
		disp_meter(temp, 'w');								// scaling, P*100 = ADC^2 * scaling / 65536	

		set_cur_lcd(LINE2);
		lcd_putst("RF Power = ");							// display attenuator cell setting
		if(cal.calval.rf_atn_bits[rf_filt_idx] == 0)
			lcd_putch(' ');									// no sign when 0 for Matti
		else
			lcd_putch('-');

		disp_bcd(cal.calval.rf_atn_bits[rf_filt_idx]);		// display attenuator bits
		lcd_putst(" dB");

// addjust
		temp = cal.calval.rf_atn_bits[rf_filt_idx];			// make int copy for +/- processing
		temp = temp - encoder_get_coarse(10);				// subtract encoder, clockwise rotation = less
		if(temp > 3)
			temp = 3;
		if(temp < 0)
			temp = 0;
		cal.calval.rf_atn_bits[rf_filt_idx] = temp;

// write to PA bosrd
		send_packet_to_74hc595();
		}

// prepare for exit exit
	save_calval();											// save calibration values on exit
	encoder_get();											// clear encoder
	}


// Handle PWR button. VFO lock, Shut down the system or RF attenuator addjust if TX && TUNE
void power_off(void)
 {
 if(tune_on != 0)														// PWR switch in tune mode takes ust to 
	addjust_rf_atttn();													// PA RF attenuator (Power level) addjustment
 else
	{

	vfo_lock = vfo_lock ^ 1;											// VFO tune lock flip-flop
	if(vfo_lock == 0)
		beep(HZ466_85, cal.calval.beep_len);							// short beep for push noticed, lock off
	else
		beep(HZ587_31, cal.calval.beep_len);							// short beep for push noticed, lock on
	clear_lcd();

	long_push = POWER_OFF;												// load power off guard timer
	while(PWR_SW == 1)
		{
		set_cur_lcd(LINE1);
		sprintf(lcdpbuff, "Pwr OFF after %i  ", (long_push/300));		// display off counter, 5,4,3,2,1,0
		lcd_putst(lcdpbuff);
		set_cur_lcd(LINE2);
		if(vfo_lock != 0)
			lcd_putst("VFO lock      ON");
		else
			lcd_putst("VFO lock     OFF");
		if(long_push == 0)
			{
			beep(HZ392_01, cal.calval.beep_len*2);						// play low sound when ready for power down
			while(PWR_SW == 1)
				{
				}
			}
		}

	if(long_push != 0)													// too short push, continue normal operation
		return;

	save_defaults();
	stop_scaf_clock();													// block audio path, reduce power off noise
	printf("\n\rUser settings saved to EEPROM, Power off\n\r");
	ms_delay(2);														// let the printout finnish
	PWR_ON = 0;
	for(;;)
		{
		}
	}
  }

// RS232 loop back test
void rs232_test(void)
	{
	unsigned char c;

	clear_lcd();
	lcd_putst("RS232 Test      ");
	set_cur_lcd(LINE2);
	lcd_putst("Waiting data....");
	for(;;)
		{
		if(kbhit())
			{
			c = getch();
			beep(HZ698_45, 10);				// give beep tone
			putch(c);						// echo character
			set_cur_lcd(LINE2);			
			lcd_putst("RX data  ");
			lcd_putch(c);					// display "as received"
			lcd_putst("  0x");
			lcd_putchhex(c);				// display in HEX format


			}

		if(PWR_SW == 1)						// exit test
			break;
		}
	}

// main __________________________________________________________________________________

int main(void)
	{
	int x, z, temp;
	unsigned int y, w;
	unsigned char c;
	int *p;
	long ltemp1, ltemp2;
	unsigned long ultemp;
	unsigned int itemp;

// set general I/O
	TRISA = INIT_TRISA;				// SWITCHES, PTT
	PORTA = INIT_PORTA;

	TRISB = INIT_TRISB;				// ADC inputs, TXEN, RXEN, General I/O
	PORTB = INIT_PORTB;

	TRISC = INIT_TRISC;				// sideband select & switches
	PORTC = INIT_PORTC;

	TRISD = INIT_TRISD;				// LEDs & LCD pins = output
	PORTD = INIT_PORTD;

	TRISF = INIT_TRISF;				// DDS controls
	PORTF = INIT_PORTF;

	TRISG = INIT_TRISG;				// DDS controls, EEPROM, DScard signals
	PORTG = INIT_PORTG;

// Power switch
	PWR_ON = 1;								// secure power ON

// start PWM system
	IPC0 =0X5444;							// set tone generator (TMR1) priority higher than others
	init_timers_pwm();						// setup pwm & tone generators

// start DDS
	dds_init();								// init AD9850
	dds_load(0);							// do first dummy load
	dds_load(DEFAULT_FREQ_A);				// dial in something, start mixer, seek operating point
	max_dds = MAX_DDS_NORMAL;				// set normal start upper limit

// CW
	cw_period = 50;							// set something before IRQ system starts
	fcy = FCY/1000;							// clock freq (kHz)

// Init Main Board SPI traffic
	rf_attn = 0;
	hc595 = 0x0000;
	init_spi1();
	exchg_data_spi1(hc595);

// start LCD
	initlcd();								// init LCD
	set_chgen();							// load bar graph fonts

// put hello messages to LCD
	clear_lcd();
	lcd_putst("JUMA-TRX2 ");				// say hello
	lcd_putst(ver);							// display version
	set_cur_lcd(LINE2);
	lcd_putst("  OH2NLT OH7SV  ");			// say who we are

// start UART & print hello messages
	InitUSART1();							// wake up serial interface

	printf("\n\r\n\rJUMA-TRX2 Firmware v1.01 / ");
	printf(ver);
	printf("\n\rCopyright Juha Niinikoski, OH2NLT\n\r\n\r");
	printf("System Clock = %i kHz\n\r", fcy);

// start ADC
	init_adc12();							// setup A/D converter, must be done before Keyer is started, RB I/O bits

// check if service mode start
	lcd_putst("Power ON");					// wait while power switch is pressed
	service_mode = 0;
	long_push = VERY_LONG_PUSH;				// set long push timer

	while(PWR_SW == 1)
		{
		if(long_push == 0)
			{
			init_scaf_clock();				// open audio channell, start SCAF clocking
			beep(HZ392_01, 300);			// give service beep
			set_cur_lcd(LINE2);	
			lcd_putst("Service Mode    ");
			service_mode = 1;
			max_dds = MAX_DDS_SERVICE;		// set service start frequency upper limit


			while(PWR_SW == 1)				// show message as logn as Power switxh is pressed
				{
				}
			set_cur_lcd(LINE1);
			lcd_putst("Push DISPLAY nxt");	// display some instructions
			set_cur_lcd(LINE2);
			lcd_putst("Push FAST = Save");
			}
		}


// Read calibration & default values from EEPROM
	y = read_calval();
	w = read_defaults();
	printf("\n\rEEPROM checksums, Cal = %X, Def Cs = %X, Factory default resets = %d",y, w, fd_counter);

	if(y != 0 || w != 0)			// compare checksums
		{
		dump_eeprom();				// test, show EEPROM content before reset
		printf("\n\rEEPROM checksum error, reload factory defaults");
		set_factory_defaults();		// EEPROM empty or corrupted, set factory defaults
		save_calval();
		save_defaults();
		}


// set LCD contrast & back light with EEPROM data
	set_pwm4_dac(eeprom.defval.contrast);
	set_pwm3_dac(eeprom.defval.back_light);

// start encoder
	encoder_init();					// init encoder, enable INT4
	vfo_lock = 0;					// tuning enabled

// set display mode etc
	lcd_mode = 0;					// select default display
	sub_page0 = 0;
	sub_page0 = 0;

	tune_on = 0;					// clear state flags
	cw_active = 0;

// set last used frequency
	vfo_idx = eeprom.defval.vfo_sel;			// copy vfo selection for processing
	if(vfo_idx > 1)								// split ?
		vfo_idx = 0;							// if split, select VFOA
	freq_in_use = eeprom.defval.vfo[vfo_idx];	// set last used frequency
	dds_load(freq_in_use);						// load DDS

// 
	ms_delay(2000);					// show start up message
	clear_lcd();

// Init ISD1700 voice memory if present
	init_isd1700();

// open audio channell and give start up notification 
	init_scaf_clock();				// open audio channell, start SCAF clocking
	ms_delay(20);
	beep(HZ698_45, cal.calval.beep_len * 2);				// give start up beep, allways

// do calibration fonctions if service mode start
	if(service_mode != 0)
		service();

// check if factory defaults asked
	while(SW5 == 0)						// if MODE button pressed ask user what to do
		{
		set_cur_lcd(LINE1);
		lcd_putst("Push SET for    ");
		set_cur_lcd(LINE2);
		lcd_putst("factory defaults");
		if(SW2 == 0)					// factory defaults asked
			{
			set_factory_defaults();		// EEPROM empty, set factory defaults
			save_calval();
			save_defaults();
			break;
			}
		}

// RS232 I/O Test
	if(SW1 == 0)						// If FUNC/DISPLAY pressed during startup
		rs232_test();					// goto RS232 test loop
				
// Main forever loop

	for(;;)
		{
// Make atomic copy of PTT_IN signal for this processing round
		if(PTT_IN_IO)
			PTT_IN = 1;						// copy I/O bit to status flag	
		else
			PTT_IN = 0;

// VFO / frequency in use select logic, A/B, Split, RX/TX

	vfo_idx = eeprom.defval.vfo_sel;		// copy vfo selection for processing, adjust selected VFO if Split


// If Split mode select VFO with RX / TX condition
		if(PTT_IN == 1)								// RX state
			{
			if(vfo_idx > 1)							// split ?
				vfo_idx = 0;						// if split,  VFOA for RX
			}
		else										// TX state
			{
			if(vfo_idx > 1)							// split ?
				{
				vfo_idx = 1;						// if split, use VFOB for TX
				}
			}			

// Evaluate CW keyer enable condition, Split mode need special conditions

		if(eeprom.defval.vfo_sel == 2)				// Split mode selected
			{
			if(eeprom.defval.mode[1] == 2)			// if TX VFO set to CW
				cw_active = 1;						// enable keyer only if TX VFO mode set to CW
			else
				cw_active = 0;
			}
		else
			{
			if(eeprom.defval.mode[vfo_idx] == 2)	// cw mode selected for currently used VFO
				cw_active = 1;
			else
				cw_active = 0;
			}

// Tune function
		if(eeprom.defval.mode[vfo_idx] == 3)			// TUNE mode selected
			{
			if(PTT_IN == 0)								// KEY follow PTT switch in tune mode
				{
				KEY = 1;
				if(tune_on == 0)
					{
					tune_on = 1;						// set tune state
					beep(HZ698_45, cal.calval.beep_len); // short 700Hz beep, enter tune state
					}
				}
			else
				{
				KEY = 0;								// remove carrier
				tune_on = 0;							// remove tune state
				}
			}
		else											// other modes
			{
			if(tune_on != 0)							// tune state should not be on in other modes
				{
				tune_on = 0;							// remove tune state
				KEY = 0;								// remove CW carrier
				} 
			}

// Handle rotary encoder ( VFO )

		if(enc_event && (lcd_mode == 0))	// adjust frequency if encoder rotated and disp page 0 selected
			{
			enc_event = 0;
			if(vfo_lock == 0)				// not locked
				{
				eeprom.defval.vfo[vfo_idx] = eeprom.defval.vfo[vfo_idx] + (encoder_get() * f_steps[eeprom.defval.step_idx]);	// get encoder +/- value
				if(eeprom.defval.vfo[vfo_idx] > max_dds)		// check tuning limits
					eeprom.defval.vfo[vfo_idx] = max_dds;
				if(eeprom.defval.vfo[vfo_idx] < MIN_DDS)
					eeprom.defval.vfo[vfo_idx] = MIN_DDS;
				}
			else
				encoder_get();				// do dummy read
			}


// Select & calculate frequency in use

			freq_in_use = eeprom.defval.vfo[vfo_idx];	// load in use frequency from selected VFO

			if(eeprom.defval.rit_on != 0)				// RIT on ?
				{
				temp = convert_adc12(RIT_POT);			// get RIT pot value
					if(temp >= (2048 + RIT_DEAD_ZONE))	// blanck dead zone
						temp = temp - (2048 + RIT_DEAD_ZONE);	// positive side
					else
						{
						if(temp <= (2048 - RIT_DEAD_ZONE))	// negative side
							temp = temp - (2048 - RIT_DEAD_ZONE);
						else
							temp = 0;					// dead zone
						}

				rit = (long)temp * RIT_SCALE;			// read RIT setting from panel potentiometer & scale it
				if(PTT_IN == 1)							// RX state
					{
					freq_in_use = freq_in_use + rit; 	// add RIT	// RIT = about +/-2kHz
					}
				}

// Handle CW shift

		if(eeprom.defval.mode[vfo_idx] == 2 && PTT_IN == 0)				// cw mode & TX
			freq_in_use = freq_in_use + eeprom.defval.cw_shift;			// +/- desired frequency

	
// Load DDS

		dds_load(freq_in_use);


// Build main board & RF filter board(only all band version) 74HC595 I/O latch data & send it
		send_packet_to_74hc595();

// select SCAF filter
		if(PTT_IN == 0 && eeprom.defval.mode[vfo_idx] < 2)						// change TX filter only when SSB and TX
			set_scaf_clock(eeprom.defval.scaf_filters[3]);						// TX, use TX  filter
		else
			set_scaf_clock(eeprom.defval.scaf_filters[eeprom.defval.filter]);	// RX, select filter	


// Convert fwd & rev A/D channells
		if(PTT_IN == 0)										// if TXon measure & scale PWR & SWR 
			{
			fwd_adval = convert_adc12(FWD_PWR);
			rev_adval = convert_adc12(REV_PWR);

// store not scaled values for later use
			fwd_pwr = (long)fwd_adval * (long)fwd_adval;	// calculate relative power levels ( U^2)
			rev_pwr = (long)rev_adval * (long)rev_adval;
			out_pwr = fwd_pwr - rev_pwr;					// output power = forward - reverse.
			swr = calc_swr(fwd_adval, rev_adval);			// calculate SWR

			if(swr_al_dly != 0)								// PTT to SWR alarm active delay
				swr_al_dly--;								// delay neded for linear amplifier switching defeat

			if(swr > SWR_ALARM && swr_al_dly == 0)			// if high SWR force SWR display on after switching defeat delay
				{
				lcd_mode = 0;								// main display
				sub_page0 = 2;								// swr display
				swr_alarm = 1;								// set alarm flag
				}
			else
				swr_alarm = 0;								// clear alarm flag
			}
		else
			swr_al_dly = SWR_AL_DLY;						// reset defeat delay counter


// set CW period

// Convert CW pot
		cw_pot = convert_adc12(CW_POT);						// cw_pot value is used also for squelch
		temp = (4096 - cw_pot);								// convert & inverse potentiometer value
#if 0
		ultemp = (unsigned long)temp * (unsigned long)temp;	// make action nonlinear, pot value^2 
		temp = ultemp >> 16;								// divide by 65536, slowest speed = 256ms
		temp = temp + 20;									// limit fastest speed to 20ms, slowest = 276ms
#endif
// alternative addjustment
		temp = temp + 340;									// min = 340, max=4435
		ultemp = (unsigned long)temp * (unsigned long)temp;	// make action nonlinear, min=115600 , max=19669225
		temp = ultemp >> 17;								// divide by 131072, slowest speed = 256ms
		temp = temp + 20;									// limit fastest speed to 20ms, slowest = 170ms

		cw_period = temp;

// Set other keyer parameters

		keyer_mode = eeprom.defval.keyer;					// copy keyer operating mode from EEPROM data


// Squelch code

	if((eeprom.defval.sq != 0) && (PTT_IN != 0))			// run squelch when squelch on and RX
		{
		if( (s_meter + 2) > (cw_pot >> 6) )					// check biased meter value against set value
			{
			start_scaf_clock();								// audio on
			sq_timer = SQ_HOLD;								// reload squelch open timer
			}
		else
			{
			if(sq_timer == 0)
				{
				stop_scaf_clock();							// audio off
				}
			}
		}

	else													// squelch not in use
		{
		start_scaf_clock();									// scaf clock on if TX or squelch not in use
		}


// Do switches

// DISPLAY / FUNC
		if(SW1 == 0)
			{
			long_push = LONG_PUSH;				// set long push timer
			if(lcd_mode == 0 && sub_page0 == MAX_SUB_PAGE0)
				beep(HZ466_85, cal.calval.beep_len);				// Bp tone for basic display
			else
				{
				if(lcd_mode == 1 && sub_page1 == MAX_SUB_PAGE1)
					beep(HZ466_85, cal.calval.beep_len);			// Bp tone for basic display
				else
					beep(HZ587_31, cal.calval.beep_len);			// D tone beep for other displays
				}

			while(SW1 == 0)
				{
				if(long_push == 0)
					{
					beep(HZ466_85, cal.calval.beep_len*10);	// long beep for major page change
					break;
					}
				}
			if(long_push == 0)										// major mode change
				{
				lcd_mode++;
				if(lcd_mode > MAX_LCD_MODE)
					lcd_mode = 0;

				while(SW1 == 0)										// wait until button released
					{
					}
				}
			else
				{
				if(lcd_mode == 0)									// select bub page to adjust
					{
					sub_page0++;
					if(sub_page0 > MAX_SUB_PAGE0)
					sub_page0 = 0;
					}
				else
					{
					sub_page1++;
					if(sub_page1 > MAX_SUB_PAGE1)
					sub_page1 = 0;					
					}
				}
			encoder_get();											// clear encoder after display change
			ms_delay(BUTTON_DEBOUNCE);
			}

// SET

		if(SW2 == 0)
			{
			if(eeprom.defval.step_idx == 2)							// current state 100 Hz
				beep(HZ466_85, cal.calval.beep_len);				// short beep, next state 10 Hz = slow
			else
				beep(HZ587_31, cal.calval.beep_len);				// short beep, next state 100Hz = fast

			long_push = LONG_PUSH;									// set long push timer
			while(SW2 == 0)
				{
				if(long_push == 0)
					{
					beep(HZ698_45, cal.calval.beep_len*10);			// long beep, fast tuning 
					break;
					}
				}
			if(long_push == 0)										// fast tuning
				{
//				beep(HZ587_31, cal.calval.beep_len*10);				// long push notification push noticed
				eeprom.defval.step_idx = 4;							// 10kHz steps
				while(SW2 == 0)										// wait for button release
					{
					}
				}
			else
				{
				if(eeprom.defval.step_idx == 2)
					eeprom.defval.step_idx = 1;	// 10Hz step
				else
					eeprom.defval.step_idx = 2;	// 100Hz step
				}
			ms_delay(BUTTON_DEBOUNCE);
			}

// RIT
		if(SW3 == 0)
			{
			if(eeprom.defval.rit_on != 0)
				beep(HZ466_85, cal.calval.beep_len);				// Bp tone beep for RIT off
			else
				beep(HZ587_31, cal.calval.beep_len);				// D tone beep for RIT on

			eeprom.defval.rit_on = 1 - eeprom.defval.rit_on; 		// Alter RIT on between 1 and 0, 1 = on
			ms_delay(BUTTON_DEBOUNCE);
			while(SW3 == 0)
				{
				}
			}

// VFO
		if(SW4 == 0 && PTT_IN != 0)									// Do not change if TX is on !
			{
			if(eeprom.defval.vfo_sel == 2)
				beep(HZ466_85, cal.calval.beep_len);				// Bp tone for VFOA
			else
				beep(HZ587_31, cal.calval.beep_len);				// D tone for VFOB & Split

//			beep(HZ466_85, cal.calval.beep_len);					// short beep for push noticed
			long_push = LONG_PUSH;									// set long push timer

			while(SW4 == 0)
				{
				if(long_push == 0)
					{
					beep(HZ466_85, cal.calval.beep_len*10);			// long beep for VFO A to B copy
					break;
					}
				}
			if(long_push == 0)										// VFO copy A to B
				{
				eeprom.defval.vfo[1] = eeprom.defval.vfo[0];
				eeprom.defval.mode[1] = eeprom.defval.mode[0];		// copy also mode
				set_cur_lcd(LINE2);									// second display line
				lcd_putst("VFOA => VFOB ok ");
				while(SW4 == 0)										// show while putton pressed
					{
					}
				}
			else
				{
				eeprom.defval.vfo_sel++;
				if(eeprom.defval.vfo_sel > 2) 						// Alter VFO A,B, Split
					eeprom.defval.vfo_sel = 0;
				}

			ms_delay(BUTTON_DEBOUNCE);
			}

// MODE
		if(SW5 == 0 && PTT_IN != 0)									// Do not change if TX is on !
			{
			if(eeprom.defval.mode[vfo_idx] == 3)					// this push returns us to LSB (0)
				beep(HZ466_85, cal.calval.beep_len);				// Bp tone for LSB
			else
				beep(HZ587_31, cal.calval.beep_len);				// D tone for others

			eeprom.defval.mode[vfo_idx]++; 							// Adjust mode, 0=LSB, 1=USB, 2=CW, 3=Tune
			if(eeprom.defval.mode[vfo_idx] > 3)
				eeprom.defval.mode[vfo_idx] = 0;
			ms_delay(BUTTON_DEBOUNCE);
			while(SW5 == 0)
				{
				}
			}

// FILTER
		if(SW6 == 0)
			{
			if(eeprom.defval.filter == 0)							// this push returns us to wide filter
				beep(HZ466_85, cal.calval.beep_len);				// Bp tone for wide
			else
				{
				if(eeprom.defval.filter == 2)						// this push returns us to mid filter
					beep(HZ587_31, cal.calval.beep_len);			// D tone for mid
				else
					beep(HZ698_45, cal.calval.beep_len);			// F tone for others(narrow)
				}

			eeprom.defval.filter--; 								// Adjust filter select, 0=nar, 1=mid, 2=wide
			if(eeprom.defval.filter < 0)
				eeprom.defval.filter = 2;
			ms_delay(BUTTON_DEBOUNCE);
			while(SW6 == 0)
				{
				}
			}


// Do LCD display

		set_cur_lcd(LINE1);										// start from line 1
		switch(lcd_mode)										// select major page to display
			{

// Normal display
			case 0: default:
				disp_bar_meter();								// display S-meter or PWR meter bar graph

// select sub mode
				switch(sub_page0)								// select sub page to display
					{
// display mode in use
					case 0: default:							// normal display
					if(eeprom.defval.mode[vfo_idx] == 0)
						{
						lcdoutch(S_BAR);						// S-meter stopper bar
						lcd_putst("LSB");
						}
					if(eeprom.defval.mode[vfo_idx] == 1)
						{
						lcdoutch(S_BAR);
						lcd_putst("USB");
						}
					if(eeprom.defval.mode[vfo_idx] == 2)
						{
						lcdoutch(S_BAR);
						lcd_putst(" CW");
						}
					if(eeprom.defval.mode[vfo_idx] == 3)
						lcd_putst("Tune");

// display filter in use
					if(PTT_IN == 0)								// TXon ?
						lcd_putst("  TX");
					else
						{
						if(eeprom.defval.filter == 0)
							lcd_putst(" NAR");
						if(eeprom.defval.filter == 1)
							lcd_putst(" MID");
						if(eeprom.defval.filter == 2)
							lcd_putst(" WID");
						}
					break;

// forward power
					case 1:										// forward power
					if(PTT_IN == 0)								// TXon ?
						{
						lcd_putst("PWR");
						temp = (out_pwr * cal.calval.fwd_pwr_mult) >> 16;
						disp_meter(temp, 'w');					// scaling, P*100 = ADC^2 * scaling / 65536		
						}
					else
						lcd_putst("PWR -.-W");
					break;

// SWR
					case 2:										// SWR
					if(PTT_IN == 0)								// TXon ?
						{
						lcd_putst("SWR");
						if(swr == 0)							// no reliable reading
							lcd_putst(" N/A ");
						else
							disp_meter(swr, 2);					// display swr	
						}
					else
						lcd_putst("SWR -.--");
					break;

// battery voltage
					case 3:										// battery voltage mean value, measure only when display selected
					batt_mean_val = batt_mean_val + convert_adc12(BATT_CH);								// measure & add
					bat_samples++;																		// add sample counter
					if(bat_samples > 15)																// 16 samples done ?
						{
						bat_samples = 0;	
						ultemp = (batt_mean_val >> 4) * cal.calval.batt_mult;							// scale Battery voltage
						batt_mean_val = 0;	
						batt = ultemp >> 8;																// div by 256, save scaled value
						lcdoutch(S_BAR);																// S-meter stopper bar
						lcd_putst(" ");
						disp_meter(batt, 2);															// display with two decimals								
						lcd_putst("V");
						}
					break;

// PA drain current
					case 4:										// Drain current, measure only when display selected
					if(PTT_IN == 0)								// TXon ?
						{
						id_cur = ((long)convert_adc12(ID_CUR) * cal.calval.id_mult) >> 16;
						lcd_putst("ID");
						disp_meter(id_cur, 2);	
						lcd_putst("A");
						}
					else
						lcd_putst("ID -.--A");
					break;

					}

				set_cur_lcd(LINE2);								// second display line
// display frequency if RIT is not on and allways when TX
				if(eeprom.defval.rit_on == 1 && PTT_IN == 1)	// RIT on and RX
					{
					vltemp = (long long)rit * (long long)cal.calval.ref_osc;	  	// calculate rit display frequency 
//					temp = vltemp >> 32;							// display true dds frequency
					temp = vltemp >> 33;							// display LO frequency = DDS freq / 2
					lcd_putst("RIT ");
					disp_rit(temp);									// special RIT display format
					}
				else												// RIT off or TX
					{
					vltemp = (long long)freq_in_use * (long long)cal.calval.ref_osc;	  	// calculate display frequency 
//					ultemp = vltemp >> 32;							// display true dds frequency
					ultemp = vltemp >> 33;							// display LO frequency = DDS freq / 2
					disp_freq(ultemp);  							// use hi-part = divided 2^32
					}

				if(vfo_lock == 0)									// not locked, normal display
					{
// display VFO
					if(eeprom.defval.vfo_sel == 0)
						lcd_putst(" VFOA");
					if(eeprom.defval.vfo_sel == 1)
						lcd_putst(" VFOB");
					if(eeprom.defval.vfo_sel > 1)					// Split
						lcd_putst(" Splt");

// display tuning step
					if(eeprom.defval.step_idx == 1)					// 10Hz step
						lcd_putst(" S");
					if(eeprom.defval.step_idx == 2)					// 100Hz step
						lcd_putst(" F");
					if(eeprom.defval.step_idx == 4)					// 10kHz step
						lcd_putst(" V");
					}

				else
					lcd_putst("   Lock");							// VFO locked

				break;					// end of main display #0


// User setup display page
			case 1:

// select sub mode
			switch(sub_page1)									// select sub page to display
					{

// AGC speed
					case 0: default:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");

					set_cur_lcd(LINE2);
					x = encoder_get();							// get encoder state & save it for evaluation
					if(x > 0)
						eeprom.defval.agc_speed = 1;			// set
					if(x < 0)
						eeprom.defval.agc_speed = 0;			// reset

					if(eeprom.defval.agc_speed != 0)
						lcd_putst("AGC = Fast      ");
					else
						lcd_putst("AGC = Slow      ");
					break;

// Adjust selected filter
					case 1:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					eeprom.defval.scaf_filters[eeprom.defval.filter] = eeprom.defval.scaf_filters[eeprom.defval.filter] + (0 - encoder_get_coarse(10));	// add changes
	
					if(eeprom.defval.scaf_filters[eeprom.defval.filter] < scaf_min[eeprom.defval.filter])				// check limits
						eeprom.defval.scaf_filters[eeprom.defval.filter] = scaf_min[eeprom.defval.filter];
					if(eeprom.defval.scaf_filters[eeprom.defval.filter] > scaf_max[eeprom.defval.filter])
						eeprom.defval.scaf_filters[eeprom.defval.filter] = scaf_max[eeprom.defval.filter];

					if(eeprom.defval.filter == 0)				// display selected filter
						lcd_putst("NAR");
					if(eeprom.defval.filter == 1)
						lcd_putst("MID");
					if(eeprom.defval.filter == 2)
						lcd_putst("WID");
					lcd_putst(" LPF  ");
					
					sprintf(lcdpbuff, "%li Hz  ", (75000L/eeprom.defval.scaf_filters[eeprom.defval.filter]));	// calculate & display corner frequency
					lcd_putst(lcdpbuff);
					break;

// Noise blancker
					case 2:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					x = encoder_get();							// get encoder state & save it for evaluation
					if(x > 0)
						eeprom.defval.nb = 1;					// set
					if(x < 0)
						eeprom.defval.nb = 0;					// reset

					if(eeprom.defval.nb != 0)
						lcd_putst("NB option = ON  ");
					else
						lcd_putst("NB option = OFF ");
					break;

// PROC on / off
					case 3:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					x = encoder_get();							// get encoder state & save it for evaluation
					if(x > 0)
						eeprom.defval.proc_on_off = 1;			// set
					if(x < 0)
						eeprom.defval.proc_on_off = 0;			// reset

					if(eeprom.defval.proc_on_off != 0)
						lcd_putst("SpeechProc = ON ");
					else
						lcd_putst("SpeechProc = OFF");
					break;

// AF Input Level
					case 4:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					x = encoder_get();							// get encoder state & save it for evaluation
					if(x > 0)
						eeprom.defval.af_input = MIC;			// select MIC level
					if(x < 0)
						eeprom.defval.af_input = LINE;			// select LINE input level

					if(eeprom.defval.af_input == LINE)			// display active selection
						lcd_putst("AF in = Line    ");
					else
						lcd_putst("AF in = Mic     ");
					break;

// Keyer operating mode
					case 5:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					eeprom.defval.keyer = eeprom.defval.keyer + encoder_get_coarse(10);		// addjust keyer operating mode
					if(eeprom.defval.keyer < 0)												// keep within limits
						eeprom.defval.keyer = 0;
					if(eeprom.defval.keyer > 3)
						eeprom.defval.keyer = 3;

					lcd_putst("Keyer = ");
					if(eeprom.defval.keyer == 0)
						lcd_putst("Dot pri.");
					if(eeprom.defval.keyer == 1)
						lcd_putst("Iambic A");
					if(eeprom.defval.keyer == 2)
						lcd_putst("Iambic B");
					if(eeprom.defval.keyer == 3)
						lcd_putst("Straight");
					break;

// LCD back light				
					case 6:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					eeprom.defval.back_light = eeprom.defval.back_light + encoder_get() * 10;
					if(eeprom.defval.back_light > MAX_BL)
						eeprom.defval.back_light = MAX_BL;
					if(eeprom.defval.back_light < 0)
						eeprom.defval.back_light = 0;
					sprintf(lcdpbuff, "LCD BL = %i      ",eeprom.defval.back_light);
					lcd_putst(lcdpbuff);
					set_pwm3_dac(eeprom.defval.back_light);
					break;

// LCD Contrast
					case 7:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					eeprom.defval.contrast = eeprom.defval.contrast + encoder_get() * 10;
					if(eeprom.defval.contrast > MAX_CONTRAST)
						eeprom.defval.contrast = MAX_CONTRAST;
					if(eeprom.defval.contrast < 0)
						eeprom.defval.contrast = 0;
					sprintf(lcdpbuff, "Contrast = %i  ",eeprom.defval.contrast);
					lcd_putst(lcdpbuff);
					set_pwm4_dac(eeprom.defval.contrast);
					break;

// Serial I/O mode select
					case 8:
					set_cur_lcd(LINE1);
					disp_bar_meter();																	// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);																	// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					eeprom.defval.serial_mode = eeprom.defval.serial_mode + encoder_get_coarse(10);		// addjust keyer operating mode
					if(eeprom.defval.serial_mode < 0)													// keep within limits
						eeprom.defval.serial_mode = 0;
					if(eeprom.defval.serial_mode > 3)
						eeprom.defval.serial_mode = 3;
					lcd_putst("RS232 = ");

					if(eeprom.defval.serial_mode == 0)
						lcd_putst("JUMATRX2");
					if(eeprom.defval.serial_mode == 1)
						lcd_putst("YaesuCAT");
					if(eeprom.defval.serial_mode == 2)
						lcd_putst("VoiceMem");
					if(eeprom.defval.serial_mode == 3)
						lcd_putst("Test    ");
					break;

// Squelch
					case 9:
					set_cur_lcd(LINE1);
					disp_bar_meter();							// display S-meter or PWR meter bar graph
					lcdoutch(S_BAR);							// S-meter stopper bar
					lcd_putst(" Adjust");
					set_cur_lcd(LINE2);

					x = encoder_get();							// get encoder state & save it for evaluation
					if(x > 0)
						eeprom.defval.sq = 1;					// set
					if(x < 0)
						eeprom.defval.sq = 0;					// reset

					if(eeprom.defval.sq != 0)
						lcd_putst("CW pot = Squelch");
					else
						lcd_putst("CW pot = CWspeed");
					break;

					}					// end switch sub_page1
				break;					// end of main disp #1

// test displays
			case 2:
				set_cur_lcd(LINE1);
				sprintf(lcdpbuff, "F %i %i W ", fwd_adval, (int)(fwd_pwr/900000));
				lcd_putst(lcdpbuff);

				set_cur_lcd(LINE2);	
				sprintf(lcdpbuff, "R %i %i W ", rev_adval, (int)(rev_pwr/900000));
				lcd_putst(lcdpbuff);
				break;					// end of main disp #2


			}							// switch(lcd_mode)  end
	

// run serial interface functions

		switch(eeprom.defval.serial_mode)
			{
			case 0:								// JUMA-TRX2 command set
				if(kbhit())
					{
					getch();
					printf("\n\rTRX_2 mode not yet implemented\n\r");
					}
			break;

			case 1:								// CAT (Yaesu) command set emulator
				serial_yaesu();
			break;

			case 2:								// External keyboard (Voice Memory buttons) handler
				serial_ext_kb();
			break;

			case 3: default:
				serial_test();					// test commands
			break;
			}


// decode PC keyboard commands


		if(PWR_SW == 1)
			power_off();						// go shut down

		}	// end for(;;)
	}		// end main
