//
// Simple UART interface for dsPIC30F6012
// Juha Niinikoski, OH2NLT 18.05.2005
//
// Below is the code to initialize the USART
// 8 bits, 1 start and 1 stop, no error handling
// kbhit and getch added 22.05.2005
// getche aded 02.06.2005unsigned char
// overrun error handling added 29.08.2005
// MPLAB C30 conversion, OH2GWE 2006.10.17


#include "juma-trx2.h"
#include <stdio.h>

//#define BAUD 9600
//#define FCY  1000000	// 4MHz XTAL
//#define FCY  8000000	// 4MHz XTAL x8 PLL = 32MHz
//#define FCY  16000000	// 4MHz XTAL x16 PLL = 64MHz
//#define FCY  29491200	// 7,3728MHz XTAL x16 PLL = 117,9648MHz


void InitUSART1(void)
{
 U1BRG = ((FCY/16)/BAUD) - 1;	// set baud rate

 U1MODE = 0x0000;
 U1STA = 0x0000;

 U1MODEbits.UARTEN = 1;					// enable uart
 U1STAbits.UTXEN = 1;					// enable transmitter
 IFS0bits.U1RXIF = 0;					// clear possible pending irq
}

// print character
void putch(char c)
	{
	while(U1STAbits.UTXBF == 1)		// wait here if tx fifo full
		{
		}
	U1TXREG = (int)c &0x00FF;	// Tx reg is 16-bit long
	}

// check if key perssed, data available from receive buffer
// Not pressed = 0, Pressed = 1
int kbhit(void)
	{
	if(U1STAbits.URXDA)
		return 1;
	else
		return 0;
	}

// receive character
unsigned char getch(void)
	{
	while(U1STAbits.URXDA == 0)		// wait for character
		{
		}
// printf("U1STAbits = 0x%x\n\r", U1STAbits); // debug
	IFS0bits.U1RXIF = 0;	// reset irq flag
	if(U1STAbits.OERR == 1)		// OERR is blocking uart
		U1STAbits.OERR = 0;		// reset overrun error if occured
	return U1RXREG;			// return with character
	}

// receive character with echo
unsigned char getche(void)
	{
	unsigned char c;
	putch(c = getch());
	return c;
	}

// Get unsigned long

unsigned long getlong(void)
	{
	unsigned long val;
	char tmp;
	val = 0;
	
	do
		{
		tmp = getch() & 0x7F;
		putch(tmp);
		if(tmp >= '0' && tmp <= '9')
			val = 10 * val + (tmp - '0');
		}
	while (tmp != 0x0D);
	
	return(val);
	}

// Special output functions

unsigned char disp_buffer[11];		// display buffer for long number printouts

/* Convert binary number to ASCII string */
/* 10 digits are converted */

void bin2ascii(unsigned long luku)
	{
char x, idx;

	idx = 9;
	for(x=0; x<10; x++)				// do 10 digits
		{
		disp_buffer[idx] = (luku % 10) + '0';
		luku = luku / 10;
		idx--;
		}
	disp_buffer[10] = 0;			// string terminator
	}

void putlong( unsigned long x)
	{
	bin2ascii(x);					// partial convert long number to string

	putch(disp_buffer[0]);		// MSB

	putch(disp_buffer[1]);		// 
	putch(disp_buffer[2]);		// 
	putch(disp_buffer[3]);		//

	putch(disp_buffer[4]);		//
	putch(disp_buffer[5]);		//
	putch(disp_buffer[6]);		//

	putch(disp_buffer[7]);		//
	putch(disp_buffer[8]);		//
	putch(disp_buffer[9]);		// LSB
	}
